#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define sbi(x,b) (x |= (1 << (b)))
#define cbi(x,b) (x &= ~(1 << (b)))

#define BAUD		115200

#define EPSILON		(0.00005)
#define PS			8
#if (PS == 1)
	#define PSOR	1
#elif (PS == 8)
	#define PSOR	2
#elif (PS == 32)
	#define PSOR	3
#elif (PS == 64)
	#define PSOR	4
#elif (PS == 128)
	#define PSOR	5
#elif (PS == 256)
	#define PSOR	6
#elif (PS == 1024)
	#define PSOR	7
#endif
#define COMP		(EPSILON * F_CPU / PS)

typedef uint32_t timeT;
typedef uint32_t* timeP;
#define TPL ((timeP) (0x100 + 20))
#define TPH ((timeP) (0x1FF - sizeof(timeT)))

volatile timeT halfMillis = 0;
volatile timeP timePointer;

inline void uart_init() {
	#define UBRR_VAL	((F_CPU+BAUD*8UL)/(BAUD*16UL)-1UL)
	UBRR0H = (uint8_t)(UBRR_VAL>>8);
	UBRR0L = (uint8_t)UBRR_VAL;
	UCSR0B = (1<<TXEN0);
}

inline void uart_transmit8(uint8_t data) {
	while ( !( UCSR0A & (1<<UDRE0)) );
	UDR0 = data;
}

inline void uart_transmit16(uint16_t data) {
	uart_transmit8((uint8_t)(data>>8));
	uart_transmit8((uint8_t)(data));
}

inline void uart_transmit32(uint32_t data) {
	uart_transmit8((uint8_t)(data>>24));
	uart_transmit8((uint8_t)(data>>16));
	uart_transmit8((uint8_t)(data>>8));
	uart_transmit8((uint8_t)(data));
}

inline void uart_transmit64(uint64_t data) {
	uart_transmit8((uint8_t)(data>>56));
	uart_transmit8((uint8_t)(data>>48));
	uart_transmit8((uint8_t)(data>>40));
	uart_transmit8((uint8_t)(data>>32));
	uart_transmit8((uint8_t)(data>>24));
	uart_transmit8((uint8_t)(data>>16));
	uart_transmit8((uint8_t)(data>>8));
	uart_transmit8((uint8_t)(data));
}

ISR(TIMER2_COMPA_vect) {
	cli();
	halfMillis++;
	sei();
}

ISR(PCINT0_vect) {
	cli();
	*timePointer = halfMillis;
	timePointer++;
	sei();
}

int main() {
	SP = 0x1FF;

	timePointer = TPL;
	while(timePointer <= TPH) {
		*timePointer = 0;
		timePointer++;
	}

	sbi(TCCR2A,WGM21);
	TCCR2B &= ~7;
	TCCR2B |= (PSOR << CS20); // Prescaler
	sbi(TIMSK2,OCIE2A);	// Compare Interrupt Timer 2
	OCR2A = COMP;

	cbi(DDRA, PINA4);
	sbi(PCICR, PCIE0);
	sbi(PCMSK0, PCINT4);

	uart_init();
	uart_transmit8(255);

	timePointer = TPL;

	sei();

	while(1) {
		if(timePointer > TPL) {
			timeP retPointer = TPL;
			while(*retPointer != 0) {
				sei();
				uart_transmit32(*retPointer);
				cli();
				*retPointer = 0;
				retPointer++;
			}
			timePointer = TPL;
			sei();
		}
		if(timePointer >= TPH)
			while(1);
	}
}
