#include <stdio.h>   /* Standard input/output definitions */
#include <stdbool.h> /* Standard boolean definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

int open_port(void) {
	int fd; /* File descriptor for the port */

	fd = open("/dev/ttyUSB0", O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd == -1)
		perror("open_port: Unable to open /dev/ttyUSB0 - ");
	else
		fcntl(fd, F_SETFL, 0);
	return (fd);
}

unsigned int uart_get32(int fd) {
	unsigned int y = 0;
	unsigned char x;
	for(x = 0; x < 4; x++) {
		unsigned char a;
		read(fd, &a, 1);
		y <<= 8;
		y |= a;
	}
	return y;
}

int main() {
	unsigned int median[67];

	FILE* fp = fopen("./parse/median.out", "r");
	int i;
	for(i = 0; i < 67; i++){
		char buffer[80];
		fgets(buffer, 80, fp);
		median[i] = atoi(buffer);
	}
	fclose(fp);

	int fd = open_port();

	struct termios options;
	tcgetattr(fd, &options);

	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);

	options.c_cflag |= (CLOCAL | CREAD);
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	tcsetattr(fd, TCSANOW, &options);

	unsigned char x;
	read(fd, &x, 1);
	printf("%d\n", x);

	unsigned int initial;
	unsigned int follow;
	unsigned int throttle;
	unsigned char chan;
	char nod;
	char gear;
	char trim;
	bool trimflag;
	char diff;
	for(;;) {
		throttle = 0;
		chan = 0;
		nod = 0;
		gear = 0;
		trim = 0;
		trimflag = false;
		initial = uart_get32(fd);
		for(i = 0; i < 67; i++) {
			follow = uart_get32(fd);
			if(i % 2 == 0) {
				diff = ((int) (follow  - initial) - (int) median[i]);
				if((diff > -5) && (diff < 5))
					diff = 0;
				else	diff = 1;
				if((i >= 4) && (i <= 16)) {
					throttle = (throttle * 2) + diff;
				} else if((i >= 18) && (i <= 24)) {
					gear = (gear * 2) + diff;
				} else if((i >= 26) && (i <= 32)) {
					nod = (nod * 2) + diff;
				} else if((i == 34) && (!diff)) {
					gear = -gear;
				} else if((i == 36) && (!diff)) {
					nod = -nod;
				} else if((i == 38) && (diff)) {
					trimflag = true;
				} else if((i >= 40) && (i <= 48)) {
					trim = (trim * 2) + diff;
				} else if((i >= 50) && (i <= 52)) {
					chan = (chan * 2) + diff;
				} 
			}
		}
		if(chan > 2) {
			printf("Fehler!\n");
			exit(1);
		}	
		if(trimflag)
			trim = -trim;
		printf("gas:%5i nicken:%5i gieren:%5i trimm:%7i channel:%c\n", throttle, nod, gear, trim, 'A' + chan);
	}
	close(fd);
}
